"""
MCP Elicitation Implementation

Implements the Elicitation API per MCP 2025-11-25 specification.
Elicitation allows servers to request additional information from users.

Features:
- Form mode: Structured data with JSON schema validation
- URL mode: External URLs for sensitive interactions
- CLI integration for interactive prompts
- Non-interactive CI mode support
"""

import logging
from dataclasses import dataclass, field
from enum import Enum
from typing import Any, Callable, Dict, List, Optional

logger = logging.getLogger(__name__)


class ElicitationMode(str, Enum):
    """Elicitation modes per MCP 2025-11-25."""
    FORM = "form"
    URL = "url"


class ElicitationAction(str, Enum):
    """Elicitation response actions per MCP 2025-11-25."""
    ACCEPT = "accept"  # User accepted and provided data
    DECLINE = "decline"  # User explicitly declined
    CANCEL = "cancel"  # User cancelled the request


# Backwards compatibility alias
class ElicitationStatus(str, Enum):
    """Elicitation response status (deprecated, use ElicitationAction)."""
    COMPLETED = "accept"  # Maps to accept
    CANCELLED = "cancel"  # Maps to cancel
    TIMEOUT = "cancel"  # Maps to cancel
    ERROR = "decline"  # Maps to decline


@dataclass
class ElicitationSchema:
    """
    Schema for form-mode elicitation.
    
    Supports JSON Schema for validation with extensions for:
    - String, number, boolean, enum types
    - Default values
    - Required fields
    """
    type: str = "object"
    properties: Dict[str, Any] = field(default_factory=dict)
    required: List[str] = field(default_factory=list)
    title: Optional[str] = None
    description: Optional[str] = None
    
    def to_dict(self) -> Dict[str, Any]:
        result = {
            "type": self.type,
            "properties": self.properties,
        }
        if self.required:
            result["required"] = self.required
        if self.title:
            result["title"] = self.title
        if self.description:
            result["description"] = self.description
        return result


@dataclass
class ElicitationRequest:
    """
    MCP Elicitation request per 2025-11-25 specification.
    
    Represents a request for user input during server operations.
    Supports form mode (structured data) and URL mode (external URLs).
    """
    message: str
    mode: ElicitationMode = ElicitationMode.FORM
    requested_schema: Optional[ElicitationSchema] = None
    url: Optional[str] = None
    elicitation_id: Optional[str] = None  # Required for URL mode
    timeout: Optional[int] = None
    metadata: Dict[str, Any] = field(default_factory=dict)
    
    # Backwards compatibility alias
    @property
    def schema(self) -> Optional[ElicitationSchema]:
        return self.requested_schema
    
    @property
    def id(self) -> str:
        return self.elicitation_id or ""
    
    def to_dict(self) -> Dict[str, Any]:
        """Convert to MCP elicitation/create params format."""
        result = {
            "message": self.message,
        }
        
        # Mode is optional for form (defaults to form if omitted)
        if self.mode == ElicitationMode.URL:
            result["mode"] = self.mode.value
        
        if self.mode == ElicitationMode.FORM and self.requested_schema:
            result["requestedSchema"] = self.requested_schema.to_dict()
        
        if self.mode == ElicitationMode.URL:
            if self.url:
                result["url"] = self.url
            if self.elicitation_id:
                result["elicitationId"] = self.elicitation_id
        
        if self.metadata:
            result["_meta"] = self.metadata
        
        return result


@dataclass
class ElicitationResult:
    """
    MCP Elicitation result per 2025-11-25 specification.
    
    Contains the user's response to an elicitation request.
    Action can be: accept, decline, or cancel.
    """
    action: ElicitationAction
    content: Optional[Dict[str, Any]] = None  # Only when action=accept
    validation_error: Optional[str] = None
    
    # Backwards compatibility aliases
    @property
    def status(self) -> ElicitationStatus:
        if self.action == ElicitationAction.ACCEPT:
            return ElicitationStatus.COMPLETED
        elif self.action == ElicitationAction.DECLINE:
            return ElicitationStatus.ERROR
        return ElicitationStatus.CANCELLED
    
    @property
    def data(self) -> Optional[Dict[str, Any]]:
        return self.content
    
    @property
    def error(self) -> Optional[str]:
        return self.validation_error
    
    def to_dict(self) -> Dict[str, Any]:
        """Convert to MCP ElicitResult format."""
        result = {
            "action": self.action.value,
        }
        
        if self.action == ElicitationAction.ACCEPT and self.content:
            result["content"] = self.content
        
        if self.validation_error:
            result["validationError"] = self.validation_error
        
        return result
    
    @classmethod
    def accept(cls, content: Dict[str, Any]) -> "ElicitationResult":
        """Create an accept result."""
        return cls(action=ElicitationAction.ACCEPT, content=content)
    
    @classmethod
    def decline(cls, error: Optional[str] = None) -> "ElicitationResult":
        """Create a decline result."""
        return cls(action=ElicitationAction.DECLINE, validation_error=error)
    
    @classmethod
    def cancel(cls) -> "ElicitationResult":
        """Create a cancel result."""
        return cls(action=ElicitationAction.CANCEL)


class ElicitationHandler:
    """
    Handles elicitation requests.
    
    Provides different handlers for:
    - Interactive CLI mode
    - Non-interactive CI mode
    - Custom handlers
    """
    
    def __init__(
        self,
        interactive: bool = True,
        default_timeout: int = 300,
        ci_mode: bool = False,
        ci_defaults: Optional[Dict[str, Any]] = None,
    ):
        """
        Initialize elicitation handler.
        
        Args:
            interactive: Whether to prompt for user input
            default_timeout: Default timeout in seconds
            ci_mode: CI mode (fail or use defaults)
            ci_defaults: Default values for CI mode
        """
        self.interactive = interactive
        self.default_timeout = default_timeout
        self.ci_mode = ci_mode
        self.ci_defaults = ci_defaults or {}
        
        self._pending_requests: Dict[str, ElicitationRequest] = {}
        self._custom_handler: Optional[Callable] = None
    
    def set_custom_handler(self, handler: Callable) -> None:
        """Set a custom elicitation handler."""
        self._custom_handler = handler
    
    async def elicit(self, request: ElicitationRequest) -> ElicitationResult:
        """
        Process an elicitation request.
        
        Args:
            request: Elicitation request
            
        Returns:
            Elicitation result
        """
        req_id = request.elicitation_id or request.id
        self._pending_requests[req_id] = request
        
        try:
            if self._custom_handler:
                return await self._custom_handler(request)
            
            if self.ci_mode:
                return await self._handle_ci_mode(request)
            
            if self.interactive:
                return await self._handle_interactive(request)
            
            return ElicitationResult.decline("No elicitation handler available")
        
        finally:
            if req_id in self._pending_requests:
                del self._pending_requests[req_id]
    
    async def _handle_ci_mode(self, request: ElicitationRequest) -> ElicitationResult:
        """Handle elicitation in CI mode."""
        if request.mode == ElicitationMode.URL:
            # URL mode cannot be handled in CI
            return ElicitationResult.decline("URL elicitation not supported in CI mode")
        
        # Try to use defaults
        if request.requested_schema:
            data = {}
            for prop_name, prop_schema in request.requested_schema.properties.items():
                if prop_name in self.ci_defaults:
                    data[prop_name] = self.ci_defaults[prop_name]
                elif "default" in prop_schema:
                    data[prop_name] = prop_schema["default"]
                elif prop_name in request.requested_schema.required:
                    return ElicitationResult.decline(
                        f"Required field '{prop_name}' has no default in CI mode"
                    )
            
            return ElicitationResult.accept(data)
        
        return ElicitationResult.accept({})
    
    async def _handle_interactive(self, request: ElicitationRequest) -> ElicitationResult:
        """Handle elicitation interactively."""
        if request.mode == ElicitationMode.URL:
            return await self._handle_url_elicitation(request)
        
        return await self._handle_form_elicitation(request)
    
    async def _handle_form_elicitation(self, request: ElicitationRequest) -> ElicitationResult:
        """Handle form-mode elicitation."""
        try:
            print(f"\n[Elicitation] {request.message}")
            
            if not request.requested_schema:
                # Simple confirmation
                response = input("Continue? [y/N]: ").strip().lower()
                if response in ("y", "yes"):
                    return ElicitationResult.accept({"confirmed": True})
                return ElicitationResult.cancel()
            
            # Collect form data
            data = {}
            for prop_name, prop_schema in request.requested_schema.properties.items():
                prop_type = prop_schema.get("type", "string")
                prop_desc = prop_schema.get("description", prop_name)
                default = prop_schema.get("default")
                required = prop_name in request.requested_schema.required
                
                prompt = f"  {prop_desc}"
                if default is not None:
                    prompt += f" [{default}]"
                if required:
                    prompt += " (required)"
                prompt += ": "
                
                # Handle enum type
                if "enum" in prop_schema:
                    options = prop_schema["enum"]
                    print(f"  Options: {', '.join(str(o) for o in options)}")
                
                value = input(prompt).strip()
                
                if not value and default is not None:
                    value = default
                elif not value and required:
                    return ElicitationResult.decline(
                        f"Required field '{prop_name}' not provided"
                    )
                
                # Type conversion
                if value:
                    if prop_type == "integer":
                        value = int(value)
                    elif prop_type == "number":
                        value = float(value)
                    elif prop_type == "boolean":
                        value = value.lower() in ("true", "yes", "1", "y")
                
                if value is not None:
                    data[prop_name] = value
            
            return ElicitationResult.accept(data)
        
        except KeyboardInterrupt:
            return ElicitationResult.cancel()
        except Exception as e:
            return ElicitationResult.decline(str(e))
    
    async def _handle_url_elicitation(self, request: ElicitationRequest) -> ElicitationResult:
        """Handle URL-mode elicitation."""
        try:
            print(f"\n[Elicitation] {request.message}")
            print(f"  Please visit: {request.url}")
            
            # Wait for user confirmation
            response = input("Press Enter when complete, or 'c' to cancel: ").strip().lower()
            
            if response == "c":
                return ElicitationResult.cancel()
            
            return ElicitationResult.accept({"url_visited": True})
        
        except KeyboardInterrupt:
            return ElicitationResult.cancel()
        except Exception as e:
            return ElicitationResult.decline(str(e))
    
    def cancel(self, request_id: str) -> bool:
        """Cancel a pending elicitation request."""
        if request_id in self._pending_requests:
            del self._pending_requests[request_id]
            return True
        return False


# Global elicitation handler
_elicitation_handler: Optional[ElicitationHandler] = None


def get_elicitation_handler() -> ElicitationHandler:
    """Get the global elicitation handler."""
    global _elicitation_handler
    if _elicitation_handler is None:
        _elicitation_handler = ElicitationHandler()
    return _elicitation_handler


def set_elicitation_handler(handler: ElicitationHandler) -> None:
    """Set the global elicitation handler."""
    global _elicitation_handler
    _elicitation_handler = handler


def create_form_request(
    message: str,
    properties: Dict[str, Any],
    required: Optional[List[str]] = None,
    title: Optional[str] = None,
    timeout: Optional[int] = None,
) -> ElicitationRequest:
    """
    Create a form-mode elicitation request per MCP 2025-11-25.
    
    Args:
        message: Message to display to user
        properties: JSON Schema properties (requestedSchema)
        required: Required field names
        title: Form title
        timeout: Request timeout
        
    Returns:
        ElicitationRequest
    """
    schema = ElicitationSchema(
        properties=properties,
        required=required or [],
        title=title,
    )
    
    return ElicitationRequest(
        message=message,
        mode=ElicitationMode.FORM,
        requested_schema=schema,
        timeout=timeout,
    )


def create_url_request(
    message: str,
    url: str,
    elicitation_id: Optional[str] = None,
    timeout: Optional[int] = None,
) -> ElicitationRequest:
    """
    Create a URL-mode elicitation request per MCP 2025-11-25.
    
    Args:
        message: Message to display to user
        url: URL to visit
        elicitation_id: Unique ID for the elicitation (auto-generated if not provided)
        timeout: Request timeout
        
    Returns:
        ElicitationRequest
    """
    import uuid
    
    return ElicitationRequest(
        message=message,
        mode=ElicitationMode.URL,
        url=url,
        elicitation_id=elicitation_id or f"elicit-{uuid.uuid4().hex[:12]}",
        timeout=timeout,
    )



"""
Fine-tuning Capabilities Module

Provides model fine-tuning functionality via LiteLLM.
"""

from dataclasses import dataclass, field
from typing import Optional, List, Any, Dict, Literal


@dataclass
class FineTuningResult:
    """Result from fine-tuning operations."""
    id: str
    object: str = "fine_tuning.job"
    model: Optional[str] = None
    status: Optional[str] = None
    training_file: Optional[str] = None
    validation_file: Optional[str] = None
    fine_tuned_model: Optional[str] = None
    created_at: Optional[int] = None
    finished_at: Optional[int] = None
    trained_tokens: Optional[int] = None
    error: Optional[Dict[str, Any]] = None
    metadata: Dict[str, Any] = field(default_factory=dict)


def fine_tuning_create(
    training_file: str,
    model: str = "gpt-4o-mini-2024-07-18",
    validation_file: Optional[str] = None,
    hyperparameters: Optional[Dict[str, Any]] = None,
    suffix: Optional[str] = None,
    custom_llm_provider: Literal["openai", "azure"] = "openai",
    timeout: float = 600.0,
    api_key: Optional[str] = None,
    api_base: Optional[str] = None,
    **kwargs
) -> FineTuningResult:
    """
    Create a fine-tuning job.
    
    Args:
        training_file: ID of the training file
        model: Base model to fine-tune
        validation_file: Optional validation file ID
        hyperparameters: Training hyperparameters
        suffix: Suffix for the fine-tuned model name
        custom_llm_provider: Provider ("openai", "azure")
        timeout: Request timeout in seconds
        api_key: Optional API key override
        api_base: Optional API base URL override
        
    Returns:
        FineTuningResult with job ID
        
    Example:
        >>> result = fine_tuning_create("file-abc123")
        >>> print(result.id, result.status)
    """
    import litellm
    
    call_kwargs = {
        'training_file': training_file,
        'model': model,
        'custom_llm_provider': custom_llm_provider,
        'timeout': timeout,
    }
    
    if validation_file:
        call_kwargs['validation_file'] = validation_file
    if hyperparameters:
        call_kwargs['hyperparameters'] = hyperparameters
    if suffix:
        call_kwargs['suffix'] = suffix
    if api_key:
        call_kwargs['api_key'] = api_key
    if api_base:
        call_kwargs['api_base'] = api_base
    
    call_kwargs.update(kwargs)
    
    response = litellm.create_fine_tuning_job(**call_kwargs)
    
    error = None
    if hasattr(response, 'error') and response.error:
        error = {
            'code': getattr(response.error, 'code', None),
            'message': getattr(response.error, 'message', None),
        }
    
    return FineTuningResult(
        id=getattr(response, 'id', ''),
        object=getattr(response, 'object', 'fine_tuning.job'),
        model=getattr(response, 'model', model),
        status=getattr(response, 'status', None),
        training_file=getattr(response, 'training_file', training_file),
        validation_file=getattr(response, 'validation_file', validation_file),
        fine_tuned_model=getattr(response, 'fine_tuned_model', None),
        created_at=getattr(response, 'created_at', None),
        finished_at=getattr(response, 'finished_at', None),
        trained_tokens=getattr(response, 'trained_tokens', None),
        error=error,
    )


async def afine_tuning_create(
    training_file: str,
    model: str = "gpt-4o-mini-2024-07-18",
    validation_file: Optional[str] = None,
    hyperparameters: Optional[Dict[str, Any]] = None,
    suffix: Optional[str] = None,
    custom_llm_provider: Literal["openai", "azure"] = "openai",
    timeout: float = 600.0,
    api_key: Optional[str] = None,
    api_base: Optional[str] = None,
    **kwargs
) -> FineTuningResult:
    """
    Async: Create a fine-tuning job.
    
    See fine_tuning_create() for full documentation.
    """
    import litellm
    
    call_kwargs = {
        'training_file': training_file,
        'model': model,
        'custom_llm_provider': custom_llm_provider,
        'timeout': timeout,
    }
    
    if validation_file:
        call_kwargs['validation_file'] = validation_file
    if hyperparameters:
        call_kwargs['hyperparameters'] = hyperparameters
    if suffix:
        call_kwargs['suffix'] = suffix
    if api_key:
        call_kwargs['api_key'] = api_key
    if api_base:
        call_kwargs['api_base'] = api_base
    
    call_kwargs.update(kwargs)
    
    response = await litellm.acreate_fine_tuning_job(**call_kwargs)
    
    error = None
    if hasattr(response, 'error') and response.error:
        error = {
            'code': getattr(response.error, 'code', None),
            'message': getattr(response.error, 'message', None),
        }
    
    return FineTuningResult(
        id=getattr(response, 'id', ''),
        object=getattr(response, 'object', 'fine_tuning.job'),
        model=getattr(response, 'model', model),
        status=getattr(response, 'status', None),
        training_file=getattr(response, 'training_file', training_file),
        validation_file=getattr(response, 'validation_file', validation_file),
        fine_tuned_model=getattr(response, 'fine_tuned_model', None),
        created_at=getattr(response, 'created_at', None),
        finished_at=getattr(response, 'finished_at', None),
        trained_tokens=getattr(response, 'trained_tokens', None),
        error=error,
    )


def fine_tuning_list(
    custom_llm_provider: Literal["openai", "azure"] = "openai",
    after: Optional[str] = None,
    limit: int = 20,
    api_key: Optional[str] = None,
    api_base: Optional[str] = None,
    **kwargs
) -> List[FineTuningResult]:
    """
    List fine-tuning jobs.
    
    Args:
        custom_llm_provider: Provider
        after: Cursor for pagination
        limit: Maximum number of jobs to return
        api_key: Optional API key override
        api_base: Optional API base URL override
        
    Returns:
        List of FineTuningResult objects
    """
    import litellm
    
    call_kwargs = {
        'custom_llm_provider': custom_llm_provider,
        'limit': limit,
    }
    
    if after:
        call_kwargs['after'] = after
    if api_key:
        call_kwargs['api_key'] = api_key
    if api_base:
        call_kwargs['api_base'] = api_base
    
    call_kwargs.update(kwargs)
    
    response = litellm.list_fine_tuning_jobs(**call_kwargs)
    
    results = []
    data = getattr(response, 'data', response) if hasattr(response, 'data') else response
    if isinstance(data, list):
        for item in data:
            error = None
            if hasattr(item, 'error') and item.error:
                error = {
                    'code': getattr(item.error, 'code', None),
                    'message': getattr(item.error, 'message', None),
                }
            
            results.append(FineTuningResult(
                id=getattr(item, 'id', ''),
                object=getattr(item, 'object', 'fine_tuning.job'),
                model=getattr(item, 'model', None),
                status=getattr(item, 'status', None),
                training_file=getattr(item, 'training_file', None),
                validation_file=getattr(item, 'validation_file', None),
                fine_tuned_model=getattr(item, 'fine_tuned_model', None),
                created_at=getattr(item, 'created_at', None),
                finished_at=getattr(item, 'finished_at', None),
                trained_tokens=getattr(item, 'trained_tokens', None),
                error=error,
            ))
    
    return results


async def afine_tuning_list(
    custom_llm_provider: Literal["openai", "azure"] = "openai",
    after: Optional[str] = None,
    limit: int = 20,
    api_key: Optional[str] = None,
    api_base: Optional[str] = None,
    **kwargs
) -> List[FineTuningResult]:
    """
    Async: List fine-tuning jobs.
    
    See fine_tuning_list() for full documentation.
    """
    import litellm
    
    call_kwargs = {
        'custom_llm_provider': custom_llm_provider,
        'limit': limit,
    }
    
    if after:
        call_kwargs['after'] = after
    if api_key:
        call_kwargs['api_key'] = api_key
    if api_base:
        call_kwargs['api_base'] = api_base
    
    call_kwargs.update(kwargs)
    
    response = await litellm.alist_fine_tuning_jobs(**call_kwargs)
    
    results = []
    data = getattr(response, 'data', response) if hasattr(response, 'data') else response
    if isinstance(data, list):
        for item in data:
            error = None
            if hasattr(item, 'error') and item.error:
                error = {
                    'code': getattr(item.error, 'code', None),
                    'message': getattr(item.error, 'message', None),
                }
            
            results.append(FineTuningResult(
                id=getattr(item, 'id', ''),
                object=getattr(item, 'object', 'fine_tuning.job'),
                model=getattr(item, 'model', None),
                status=getattr(item, 'status', None),
                training_file=getattr(item, 'training_file', None),
                validation_file=getattr(item, 'validation_file', None),
                fine_tuned_model=getattr(item, 'fine_tuned_model', None),
                created_at=getattr(item, 'created_at', None),
                finished_at=getattr(item, 'finished_at', None),
                trained_tokens=getattr(item, 'trained_tokens', None),
                error=error,
            ))
    
    return results



"""
MCP (Model Context Protocol) Capabilities Module

Provides MCP server interaction functionality.
"""

from dataclasses import dataclass, field
from typing import Optional, Any, Dict, List


@dataclass
class MCPResult:
    """Result from MCP operations."""
    tools: Optional[List[Dict[str, Any]]] = None
    resources: Optional[List[Dict[str, Any]]] = None
    prompts: Optional[List[Dict[str, Any]]] = None
    server_name: Optional[str] = None
    metadata: Dict[str, Any] = field(default_factory=dict)


@dataclass
class MCPToolCallResult:
    """Result from MCP tool call."""
    result: Any
    tool_name: str
    server_name: Optional[str] = None
    metadata: Dict[str, Any] = field(default_factory=dict)


def mcp_list_tools(
    command: str,
    args: Optional[List[str]] = None,
    env: Optional[Dict[str, str]] = None,
    timeout: float = 30.0,
    metadata: Optional[Dict[str, Any]] = None,
    **kwargs
) -> MCPResult:
    """
    List tools from an MCP server.
    
    Args:
        command: MCP server command (e.g., "npx")
        args: Command arguments
        env: Environment variables
        timeout: Request timeout in seconds
        metadata: Optional metadata for tracing
        
    Returns:
        MCPResult with available tools
        
    Example:
        >>> result = mcp_list_tools("npx", ["-y", "@modelcontextprotocol/server-filesystem", "."])
        >>> for tool in result.tools:
        ...     print(tool['name'])
    """
    try:
        from praisonaiagents.mcp import MCP
        
        mcp = MCP(command=command, args=args or [], env=env)
        tools = mcp.get_tools()
        
        tool_list = []
        for tool in tools:
            tool_list.append({
                'name': getattr(tool, 'name', str(tool)),
                'description': getattr(tool, 'description', ''),
            })
        
        return MCPResult(
            tools=tool_list,
            server_name=command,
            metadata=metadata or {},
        )
    except ImportError:
        return MCPResult(
            tools=[],
            server_name=command,
            metadata={"error": "MCP not available", **(metadata or {})},
        )


async def amcp_list_tools(
    command: str,
    args: Optional[List[str]] = None,
    env: Optional[Dict[str, str]] = None,
    timeout: float = 30.0,
    metadata: Optional[Dict[str, Any]] = None,
    **kwargs
) -> MCPResult:
    """
    Async: List tools from an MCP server.
    
    See mcp_list_tools() for full documentation.
    """
    return mcp_list_tools(
        command=command,
        args=args,
        env=env,
        timeout=timeout,
        metadata=metadata,
        **kwargs
    )


def mcp_call_tool(
    command: str,
    tool_name: str,
    tool_args: Optional[Dict[str, Any]] = None,
    args: Optional[List[str]] = None,
    env: Optional[Dict[str, str]] = None,
    timeout: float = 60.0,
    metadata: Optional[Dict[str, Any]] = None,
    **kwargs
) -> MCPToolCallResult:
    """
    Call a tool on an MCP server.
    
    Args:
        command: MCP server command
        tool_name: Name of the tool to call
        tool_args: Arguments for the tool
        args: Command arguments
        env: Environment variables
        timeout: Request timeout in seconds
        metadata: Optional metadata for tracing
        
    Returns:
        MCPToolCallResult with tool result
    """
    try:
        from praisonaiagents.mcp import MCP
        
        mcp = MCP(command=command, args=args or [], env=env)
        
        # Get the tool and call it
        tools = mcp.get_tools()
        for tool in tools:
            if getattr(tool, 'name', str(tool)) == tool_name:
                if callable(tool):
                    result = tool(**(tool_args or {}))
                else:
                    result = None
                
                return MCPToolCallResult(
                    result=result,
                    tool_name=tool_name,
                    server_name=command,
                    metadata=metadata or {},
                )
        
        return MCPToolCallResult(
            result=None,
            tool_name=tool_name,
            server_name=command,
            metadata={"error": f"Tool '{tool_name}' not found", **(metadata or {})},
        )
    except ImportError:
        return MCPToolCallResult(
            result=None,
            tool_name=tool_name,
            server_name=command,
            metadata={"error": "MCP not available", **(metadata or {})},
        )


async def amcp_call_tool(
    command: str,
    tool_name: str,
    tool_args: Optional[Dict[str, Any]] = None,
    args: Optional[List[str]] = None,
    env: Optional[Dict[str, str]] = None,
    timeout: float = 60.0,
    metadata: Optional[Dict[str, Any]] = None,
    **kwargs
) -> MCPToolCallResult:
    """
    Async: Call a tool on an MCP server.
    
    See mcp_call_tool() for full documentation.
    """
    return mcp_call_tool(
        command=command,
        tool_name=tool_name,
        tool_args=tool_args,
        args=args,
        env=env,
        timeout=timeout,
        metadata=metadata,
        **kwargs
    )



"""
Rich/prompt_toolkit frontend for InteractiveCore.

This frontend provides a REPL-style interface using Rich for rendering
and prompt_toolkit for input handling.
"""

import asyncio
import logging
from typing import Optional

from ..core import InteractiveCore
from ..config import InteractiveConfig
from ..events import (
    InteractiveEvent,
    InteractiveEventType,
    ApprovalRequest,
    ApprovalResponse,
    ApprovalDecision,
)

logger = logging.getLogger(__name__)


class RichFrontend:
    """
    Rich/prompt_toolkit frontend for interactive mode.
    
    This is used by:
    - `praisonai run --interactive`
    - `praisonai chat`
    """
    
    def __init__(self, core: Optional[InteractiveCore] = None, config: Optional[InteractiveConfig] = None):
        """Initialize the Rich frontend.
        
        Args:
            core: InteractiveCore instance. If None, creates one.
            config: Configuration. Used if core is None.
        """
        self.core = core or InteractiveCore(config=config)
        self._running = False
        self._current_response = ""
        self._pending_approval: Optional[ApprovalRequest] = None
        self._approval_event = asyncio.Event()
        self._approval_response: Optional[ApprovalResponse] = None
        
        # Subscribe to events
        self.core.subscribe(self._handle_event)
    
    def _handle_event(self, event: InteractiveEvent) -> None:
        """Handle events from InteractiveCore."""
        if event.type == InteractiveEventType.MESSAGE_START:
            self._on_message_start(event)
        elif event.type == InteractiveEventType.MESSAGE_CHUNK:
            self._on_message_chunk(event)
        elif event.type == InteractiveEventType.MESSAGE_END:
            self._on_message_end(event)
        elif event.type == InteractiveEventType.TOOL_START:
            self._on_tool_start(event)
        elif event.type == InteractiveEventType.TOOL_END:
            self._on_tool_end(event)
        elif event.type == InteractiveEventType.APPROVAL_ASKED:
            self._on_approval_asked(event)
        elif event.type == InteractiveEventType.ERROR:
            self._on_error(event)
        elif event.type == InteractiveEventType.SESSION_CREATED:
            self._on_session_created(event)
        elif event.type == InteractiveEventType.SESSION_RESUMED:
            self._on_session_resumed(event)
    
    def _on_message_start(self, event: InteractiveEvent) -> None:
        """Handle message start."""
        try:
            from rich.console import Console
            console = Console()
            console.print("\n[bold cyan]Assistant:[/bold cyan]", end=" ")
        except ImportError:
            print("\nAssistant:", end=" ")
    
    def _on_message_chunk(self, event: InteractiveEvent) -> None:
        """Handle message chunk (streaming)."""
        chunk = event.data.get("chunk", "")
        self._current_response += chunk
        print(chunk, end="", flush=True)
    
    def _on_message_end(self, event: InteractiveEvent) -> None:
        """Handle message end."""
        response = event.data.get("response", "")
        if response and not self._current_response:
            # Full response (non-streaming)
            print(response)
        else:
            print()  # Newline after streaming
        self._current_response = ""
    
    def _on_tool_start(self, event: InteractiveEvent) -> None:
        """Handle tool execution start."""
        tool_name = event.data.get("tool_name", "unknown")
        try:
            from rich.console import Console
            console = Console()
            console.print(f"[dim]⚙ Running {tool_name}...[/dim]")
        except ImportError:
            print(f"⚙ Running {tool_name}...")
    
    def _on_tool_end(self, event: InteractiveEvent) -> None:
        """Handle tool execution end."""
        tool_name = event.data.get("tool_name", "unknown")
        success = event.data.get("success", True)
        
        try:
            from rich.console import Console
            console = Console()
            if success:
                console.print(f"[dim]✓ {tool_name} completed[/dim]")
            else:
                console.print(f"[red]✗ {tool_name} failed[/red]")
        except ImportError:
            status = "✓" if success else "✗"
            print(f"{status} {tool_name} {'completed' if success else 'failed'}")
    
    def _on_approval_asked(self, event: InteractiveEvent) -> None:
        """Handle approval request."""
        self._pending_approval = ApprovalRequest(**event.data)
    
    def _on_error(self, event: InteractiveEvent) -> None:
        """Handle error."""
        error = event.data.get("error", "Unknown error")
        try:
            from rich.console import Console
            console = Console()
            console.print(f"[bold red]Error:[/bold red] {error}")
        except ImportError:
            print(f"Error: {error}")
    
    def _on_session_created(self, event: InteractiveEvent) -> None:
        """Handle session created."""
        session_id = event.data.get("session_id", "")
        try:
            from rich.console import Console
            console = Console()
            console.print(f"[dim]Session created: {session_id[:8]}...[/dim]")
        except ImportError:
            print(f"Session created: {session_id[:8]}...")
    
    def _on_session_resumed(self, event: InteractiveEvent) -> None:
        """Handle session resumed."""
        session_id = event.data.get("session_id", "")
        try:
            from rich.console import Console
            console = Console()
            console.print(f"[dim]Session resumed: {session_id[:8]}...[/dim]")
        except ImportError:
            print(f"Session resumed: {session_id[:8]}...")
    
    def _prompt_approval(self, request: ApprovalRequest) -> ApprovalResponse:
        """Prompt user for approval decision."""
        try:
            from rich.console import Console
            from rich.panel import Panel
            console = Console()
            
            console.print(Panel(
                f"[bold]{request.description}[/bold]\n\n"
                f"Tool: {request.tool_name}\n"
                f"Action: {request.action_type}",
                title="[yellow]Approval Required[/yellow]",
                border_style="yellow"
            ))
            
            console.print("[1] Allow once")
            console.print("[2] Always allow this pattern")
            console.print("[3] Always allow for this session")
            console.print("[4] Reject")
            
        except ImportError:
            print(f"\n=== Approval Required ===")
            print(f"Description: {request.description}")
            print(f"Tool: {request.tool_name}")
            print(f"Action: {request.action_type}")
            print("[1] Allow once")
            print("[2] Always allow this pattern")
            print("[3] Always allow for this session")
            print("[4] Reject")
        
        while True:
            try:
                choice = input("\nChoice [1-4]: ").strip()
                
                if choice == "1":
                    return ApprovalResponse(
                        request_id=request.request_id,
                        decision=ApprovalDecision.ONCE
                    )
                elif choice == "2":
                    pattern = f"{request.action_type}:*"
                    return ApprovalResponse(
                        request_id=request.request_id,
                        decision=ApprovalDecision.ALWAYS,
                        remember_pattern=pattern
                    )
                elif choice == "3":
                    pattern = f"{request.action_type}:*"
                    return ApprovalResponse(
                        request_id=request.request_id,
                        decision=ApprovalDecision.ALWAYS_SESSION,
                        remember_pattern=pattern
                    )
                elif choice == "4":
                    return ApprovalResponse(
                        request_id=request.request_id,
                        decision=ApprovalDecision.REJECT
                    )
                else:
                    print("Invalid choice. Please enter 1-4.")
            except (EOFError, KeyboardInterrupt):
                return ApprovalResponse(
                    request_id=request.request_id,
                    decision=ApprovalDecision.REJECT
                )
    
    async def run(self) -> None:
        """Run the interactive REPL loop."""
        self._running = True
        
        try:
            from rich.console import Console
            console = Console()
            console.print("[bold green]PraisonAI Interactive Mode[/bold green]")
            console.print("[dim]Type /help for commands, /exit to quit[/dim]\n")
        except ImportError:
            print("PraisonAI Interactive Mode")
            print("Type /help for commands, /exit to quit\n")
        
        # Handle --continue flag
        if self.core.config.continue_session:
            session_id = self.core.continue_session()
            if not session_id:
                try:
                    from rich.console import Console
                    Console().print("[yellow]No previous session found. Starting new session.[/yellow]")
                except ImportError:
                    print("No previous session found. Starting new session.")
        
        # Try to use prompt_toolkit for better input
        try:
            from prompt_toolkit import PromptSession
            from prompt_toolkit.history import FileHistory
            from pathlib import Path
            
            history_file = Path.home() / ".praison" / "history"
            history_file.parent.mkdir(parents=True, exist_ok=True)
            
            session = PromptSession(history=FileHistory(str(history_file)))
            get_input = lambda: session.prompt("You: ")
        except ImportError:
            get_input = lambda: input("You: ")
        
        while self._running:
            try:
                user_input = get_input().strip()
                
                if not user_input:
                    continue
                
                # Handle slash commands
                if user_input.startswith("/"):
                    if await self._handle_command(user_input):
                        continue
                
                # Execute prompt
                await self.core.prompt(user_input)
                
            except (EOFError, KeyboardInterrupt):
                print("\nGoodbye!")
                break
            except Exception as e:
                logger.exception("Error in REPL loop")
                try:
                    from rich.console import Console
                    Console().print(f"[red]Error: {e}[/red]")
                except ImportError:
                    print(f"Error: {e}")
    
    async def _handle_command(self, command: str) -> bool:
        """Handle slash commands. Returns True if command was handled."""
        cmd = command.lower().split()[0]
        args = command.split()[1:] if len(command.split()) > 1 else []
        
        try:
            from rich.console import Console
            console = Console()
        except ImportError:
            console = None
        
        if cmd in ("/exit", "/quit", "/q"):
            self._running = False
            return True
        
        elif cmd == "/help":
            help_text = """
Available commands:
  /help          - Show this help
  /exit, /quit   - Exit interactive mode
  /clear         - Clear conversation history
  /session       - Show current session info
  /sessions      - List all sessions
  /continue      - Continue last session
  /export <file> - Export session to file
  /import <file> - Import session from file
  /model <name>  - Switch model
  /cost          - Show token/cost usage
"""
            if console:
                console.print(help_text)
            else:
                print(help_text)
            return True
        
        elif cmd == "/clear":
            # Create new session
            self.core.create_session()
            if console:
                console.print("[dim]Conversation cleared.[/dim]")
            else:
                print("Conversation cleared.")
            return True
        
        elif cmd == "/session":
            session_id = self.core.current_session_id
            if session_id:
                if console:
                    console.print(f"[dim]Current session: {session_id}[/dim]")
                else:
                    print(f"Current session: {session_id}")
            else:
                if console:
                    console.print("[dim]No active session[/dim]")
                else:
                    print("No active session")
            return True
        
        elif cmd == "/sessions":
            sessions = self.core.session_store.list_sessions()
            if sessions:
                if console:
                    console.print("[bold]Sessions:[/bold]")
                    for s in sessions[:10]:
                        sid = s.get("session_id", "")[:8]
                        title = s.get("title", "Untitled")
                        console.print(f"  {sid}... - {title}")
                else:
                    print("Sessions:")
                    for s in sessions[:10]:
                        print(f"  {s.get('session_id', '')[:8]}... - {s.get('title', 'Untitled')}")
            else:
                if console:
                    console.print("[dim]No sessions found[/dim]")
                else:
                    print("No sessions found")
            return True
        
        elif cmd == "/continue":
            session_id = self.core.continue_session()
            if session_id:
                if console:
                    console.print(f"[dim]Continued session: {session_id[:8]}...[/dim]")
                else:
                    print(f"Continued session: {session_id[:8]}...")
            else:
                if console:
                    console.print("[yellow]No previous session found[/yellow]")
                else:
                    print("No previous session found")
            return True
        
        elif cmd == "/export":
            if not args:
                if console:
                    console.print("[red]Usage: /export <filename>[/red]")
                else:
                    print("Usage: /export <filename>")
                return True
            
            session_id = self.core.current_session_id
            if session_id:
                try:
                    self.core.export_session_to_file(session_id, args[0])
                    if console:
                        console.print(f"[green]Session exported to {args[0]}[/green]")
                    else:
                        print(f"Session exported to {args[0]}")
                except Exception as e:
                    if console:
                        console.print(f"[red]Export failed: {e}[/red]")
                    else:
                        print(f"Export failed: {e}")
            else:
                if console:
                    console.print("[yellow]No active session to export[/yellow]")
                else:
                    print("No active session to export")
            return True
        
        elif cmd == "/import":
            if not args:
                if console:
                    console.print("[red]Usage: /import <filename>[/red]")
                else:
                    print("Usage: /import <filename>")
                return True
            
            try:
                session_id = self.core.import_session_from_file(args[0])
                self.core.resume_session(session_id)
                if console:
                    console.print(f"[green]Session imported: {session_id[:8]}...[/green]")
                else:
                    print(f"Session imported: {session_id[:8]}...")
            except Exception as e:
                if console:
                    console.print(f"[red]Import failed: {e}[/red]")
                else:
                    print(f"Import failed: {e}")
            return True
        
        elif cmd == "/model":
            if args:
                self.core.config.model = args[0]
                if console:
                    console.print(f"[dim]Model set to: {args[0]}[/dim]")
                else:
                    print(f"Model set to: {args[0]}")
            else:
                model = self.core.config.model or "default"
                if console:
                    console.print(f"[dim]Current model: {model}[/dim]")
                else:
                    print(f"Current model: {model}")
            return True
        
        elif cmd == "/cost":
            # TODO: Integrate with cost tracker
            if console:
                console.print("[dim]Cost tracking not yet implemented in unified core[/dim]")
            else:
                print("Cost tracking not yet implemented in unified core")
            return True
        
        return False  # Command not recognized
    
    def stop(self) -> None:
        """Stop the REPL loop."""
        self._running = False
